@@ -683,13 +683,6 @@ int
 main(int argc, char *argv[])
 {
     debug("ntlm_auth build " __DATE__ ", " __TIME__ " starting up...\n");
-#if DEBUG
-    debug("changing dir to /tmp\n");
-    if (chdir("/tmp") != 0) {
-        debug("ERROR: (%d) failed.\n",errno);
-        return 2;
-    }
-#endif
 
     my_program_name = argv[0];
     process_options(argc, argv);
