@@ -238,7 +238,7 @@ typedef enum {
 /// \ingroup CBDATAAPI
 extern void cbdataRegisterWithCacheManager(void);
 
-#if CBDATA_DEBUG
+#if USE_CBDATA_DEBUG
 extern void *cbdataInternalAllocDbg(cbdata_type type, const char *, int);
 extern void *cbdataInternalFreeDbg(void *p, const char *, int);
 extern void cbdataInternalLockDbg(const void *p, const char *, int);
@@ -278,13 +278,14 @@ extern cbdata_type cbdataInternalAddType(cbdata_type type, const char *label, in
 
 
 /* cbdata macros */
-#if CBDATA_DEBUG
+#if USE_CBDATA_DEBUG
 #define cbdataAlloc(type)	((type *)cbdataInternalAllocDbg(CBDATA_##type,__FILE__,__LINE__))
 #define cbdataFree(var)		do {if (var) {cbdataInternalFreeDbg(var,__FILE__,__LINE__); var = NULL;}} while(0)
 #define cbdataInternalLock(a)		cbdataInternalLockDbg(a,__FILE__,__LINE__)
 #define cbdataInternalUnlock(a)		cbdataInternalUnlockDbg(a,__FILE__,__LINE__)
 #define cbdataReferenceValidDone(var, ptr) cbdataInternalReferenceDoneValidDbg((void **)&(var), (ptr), __FILE__,__LINE__)
 #define CBDATA_CLASS2(type)	\
+	private: \
 	static cbdata_type CBDATA_##type; \
 	public: \
 		void *operator new(size_t size) { \
@@ -295,9 +296,8 @@ extern cbdata_type cbdataInternalAddType(cbdata_type type, const char *label, in
   		void operator delete (void *address) { \
 		  if (address) cbdataInternalFreeDbg(address,__FILE__,__LINE__); \
 		} \
-                void *toCbdata() { return this; } \
-	private:
-#else
+                void *toCbdata() { return this; }
+#else /* USE_CBDATA_DEBUG */
 
 /**
  \ingroup CBDATAAPI
@@ -448,7 +448,7 @@ class generic_cbdata
 {
 public:
 
-    generic_cbdata(void * data) : data(data) {}
+    generic_cbdata(void * aData) : data(aData) {}
 
     template<typename wrapped_type>void unwrap(wrapped_type **output) {
         *output = static_cast<wrapped_type *>(data);
