@@ -35,7 +35,7 @@
 
 #include "squid.h"
 
-#if USE_ICMP && USE_IPV6
+#if USE_ICMP
 
 #include "SquidTime.h"
 #include "Debug.h"
@@ -124,7 +124,7 @@ Icmp6::Open(void)
     }
 
     icmp_ident = getpid() & 0xffff;
-    debugs(42, 1, "pinger: Icmp6 socket opened");
+    debugs(42, 1, "pinger: ICMPv6 socket opened");
 
     return icmp_sock;
 }
@@ -133,7 +133,7 @@ Icmp6::Open(void)
  * Generates an RFC 4443 Icmp6 ECHO Packet and sends into the network.
  */
 void
-Icmp6::SendEcho(IpAddress &to, int opcode, const char *payload, int len)
+Icmp6::SendEcho(Ip::Address &to, int opcode, const char *payload, int len)
 {
     int x;
     LOCAL_ARRAY(char, pkt, MAX_PKT6_SZ);
@@ -174,7 +174,7 @@ Icmp6::SendEcho(IpAddress &to, int opcode, const char *payload, int len)
         if (len > MAX_PAYLOAD)
             len = MAX_PAYLOAD;
 
-        xmemcpy(echo->payload, payload, len);
+        memcpy(echo->payload, payload, len);
 
         icmp6_pktsize += len;
     }
@@ -213,7 +213,7 @@ Icmp6::Recv(void)
     struct addrinfo *from = NULL;
 //    struct ip6_hdr *ip = NULL;
     static char *pkt = NULL;
-    struct icmp6_hdr *icmp6 = NULL;
+    struct icmp6_hdr *icmp6header = NULL;
     icmpEchoData *echo = NULL;
     struct timeval now;
     static pingerReplyData preply;
@@ -278,12 +278,12 @@ Icmp6::Recv(void)
     );
     */
 
-    icmp6 = (struct icmp6_hdr *) pkt;
+    icmp6header = (struct icmp6_hdr *) pkt;
     pkt += sizeof(icmp6_hdr);
 
-    if (icmp6->icmp6_type != ICMP6_ECHO_REPLY) {
+    if (icmp6header->icmp6_type != ICMP6_ECHO_REPLY) {
 
-        switch (icmp6->icmp6_type) {
+        switch (icmp6header->icmp6_type) {
         case 134:
         case 135:
         case 136:
@@ -291,15 +291,15 @@ Icmp6::Recv(void)
             break;
 
         default:
-            debugs(42, 8, HERE << preply.from << " said: " << icmp6->icmp6_type << "/" << (int)icmp6->icmp6_code << " " <<
-                   ( icmp6->icmp6_type&0x80 ? icmp6HighPktStr[(int)(icmp6->icmp6_type&0x7f)] : icmp6LowPktStr[(int)(icmp6->icmp6_type&0x7f)] )
+            debugs(42, 8, HERE << preply.from << " said: " << icmp6header->icmp6_type << "/" << (int)icmp6header->icmp6_code << " " <<
+                   ( icmp6header->icmp6_type&0x80 ? icmp6HighPktStr[(int)(icmp6header->icmp6_type&0x7f)] : icmp6LowPktStr[(int)(icmp6header->icmp6_type&0x7f)] )
                   );
         }
         return;
     }
 
-    if (icmp6->icmp6_id != icmp_ident) {
-        debugs(42, 8, HERE << "dropping Icmp6 read. IDENT check failed. ident=='" << icmp_ident << "'=='" << icmp6->icmp6_id << "'");
+    if (icmp6header->icmp6_id != icmp_ident) {
+        debugs(42, 8, HERE << "dropping Icmp6 read. IDENT check failed. ident=='" << icmp_ident << "'=='" << icmp6header->icmp6_id << "'");
         return;
     }
 
@@ -329,8 +329,8 @@ Icmp6::Recv(void)
     }
 
     Log(preply.from,
-        icmp6->icmp6_type,
-        ( icmp6->icmp6_type&0x80 ? icmp6HighPktStr[(int)(icmp6->icmp6_type&0x7f)] : icmp6LowPktStr[(int)(icmp6->icmp6_type&0x7f)] ),
+        icmp6header->icmp6_type,
+        ( icmp6header->icmp6_type&0x80 ? icmp6HighPktStr[(int)(icmp6header->icmp6_type&0x7f)] : icmp6LowPktStr[(int)(icmp6header->icmp6_type&0x7f)] ),
         preply.rtt,
         preply.hops);
 
@@ -338,4 +338,4 @@ Icmp6::Recv(void)
     control.SendResult(preply, (sizeof(pingerReplyData) - PINGER_PAYLOAD_SZ + preply.psize) );
 }
 
-#endif /* USE_ICMP && USE_IPV6 */
+#endif /* USE_ICMP */
