@@ -126,7 +126,7 @@ using namespace Squid;
 #if HAVE_LIMITS_H
 #include <limits.h>
 #endif
-#ifdef _SQUID_WIN32_
+#if _SQUID_WINDOWS_
 #include <io.h>
 #endif
 #if HAVE_SYS_MOUNT_H
@@ -154,9 +154,6 @@ using namespace Squid;
 #endif
 
 #include "md5.h"
-#if USE_SSL
-#include "ssl_support.h"
-#endif
 #if SQUID_SNMP
 #include "cache_snmp.h"
 #endif
